/*
 * Copyright 2005,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ini4j.sample;

//<editor-fold defaultstate="collapsed" desc="apt documentation">
//|
//|                ---------------
//|                Dwarfs interface
//|
//|Dwarfs interface
//|
//</editor-fold>
//{
public interface Dwarfs
{
    String PROP_BASHFUL = "bashful";
    String PROP_DOC = "doc";
    String PROP_DOPEY = "dopey";
    String PROP_GRUMPY = "grumpy";
    String PROP_HAPPY = "happy";
    String PROP_SLEEPY = "sleepy";
    String PROP_SNEEZY = "sneezy";

    Dwarf getBashful();

    Dwarf getDoc();

    Dwarf getDopey();

    Dwarf getGrumpy();

    Dwarf getHappy();

    Dwarf getSleepy();

    Dwarf getSneezy();
}
//}
