/*
 * Copyright 2005,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ini4j.sample;
//|
//|                -------
//|                Samples
//|
//|Samples
//|
//| The simple (but useful) samples you can find here illustrate the usage of [ini4j].
//| These are small, independent programs that may be translates and run.
//| The build process of [ini4j] guarantees compilability and runability, since no
//| distribution may be created without compiling and running these programs.
//|
