/*
 * Copyright 2005,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ini4j.spi;

import org.ini4j.Registry.Type;

public class TypeValuesPair
{
    private final Type _type;
    private final String[] _values;

    @SuppressWarnings("PMD.ArrayIsStoredDirectly")
    public TypeValuesPair(Type type, String[] values)
    {
        _type = type;
        _values = values;
    }

    public Type getType()
    {
        return _type;
    }

    public String[] getValues()
    {
        return _values;
    }
}
