/*
 * Copyright 2005,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
h1,h2,h3,h4,h5,h6
{
    border: none;
/*    border-bottom: 1px solid #808080; */
    background-color: white;
    font-family: Palatino, Linotype, Georgia, "Times New Roman", Times, serif;
    font-weight: normal;
}

h1,h2
{
   /* border-bottom: 3px double #808080;*/
}

#breadcrumbs
{
    border: none;
    background-color: #f0f0f0;
    border-top: 1px solid #a0a0a0;
    color: #c0c0c0;
    /*border-bottom: 1px solid #a0a0a0;*/
}

a:link
{
    color: #409040;
}

a:visited
{
    color: #408040;
}

a:hover, a:active
{
    color: #20c020;
}

#footer
{
    border-top: 1px solid #e0e0e0;
    color: #a0a0a0;
}

.source
{
    background-color: #fffff0;
}

#leftColumn
{
    border: none;
    background-color: white;
}

#navcolumn h5
{
    border: none;
    font-family: Verdana, Helvetica, Arial, sans-serif;
}

#bodyColumn
{
    margin-left: 170px;
}

#leftcol, .docs #toc, .docs .courtesylinks, #leftColumn, #navColumn {
/*	display: none !important;*/
}
#bodyColumn, body.docs div.docs {
	/*margin: 0 !important;
	border: none !important;*/
}

menu li
{
   list-style: none;
   display: inline;
   padding: 10px;
}

menu li img
{
   vertical-align: middle;
}

menu
{
    padding-top: 20px;
}

table
{
    width: 99%;
}
